MODULE crs_iom
   !!=====================================================================
   !!                    ***  MODULE  crs_iom ***
   !! Input/Output manager :  Library to read input files with NF90 (only fliocom module)
   !!====================================================================
   !! History :  9.0  ! 12 06  (J. Simeon) Original code
   !!
   !!
   !!            kdompar_crs(1,1) = global x-dimension, jpiglo_crs
   !!            kdompar_crs(2,1) = global y-dimension, jpjglo_crs
   !!            kdompar_crs(:,2)  
   !!  from dom_oce INTEGER nimpp, njmpp      !: i- & j-indexes for mpp-subdomain left bottom
   !!
   !!  From LBC/mppini.F90      
   !!  ! No mpp computation
   !!   nimpp  = 1
   !!   njmpp  = 1
   !!   nlci   = jpi
   !!   nlcj   = jpj
   !!   nldi   = 1
   !!   nldj   = 1
   !!   nlei   = jpi
   !!   nlej   = jpj
   !!   nperio = jperio
   !!   nbondi = 2
   !!   nbondj = 2
   !!   nidom  = FLIO_DOM_NONE
   !!   npolj = jperio
   !!
   !! jes. 28 Jun 2012. TODO. make sure of variable declarations to be placed here or crs_dom.F90
   !!--------------------------------------------------------------------
   USE timing
   USE crs_dom
   USE dom_oce         ! ocean space and time domain
   USE iom_def         ! iom variables definitions
   USE netcdf          ! NetCDF library
   USE in_out_manager  ! I/O manager
   USE lib_mpp         ! MPP library
   USE iom             ! I/O library
   USE par_kind, ONLY: wp   


   IMPLICIT NONE
   PRIVATE

   PUBLIC crs_iom_open, crs_iom_close, crs_iom_rstput, crs_iom_put

!   PUBLIC crs_iom_varid, crs_iom_get, crs_iom_gettime


   INTEGER, PARAMETER ::   jpdomcrs_data          = 1   !: ( 1  :jpiglo_crs, 1  :jpjglo_crs)
   INTEGER, PARAMETER ::   jpdomcrs_global        = 2   !: ( 1  :jpiglo_crs, 1  :jpjglo_crs)
   INTEGER, PARAMETER ::   jpdomcrs_local         = 3   !: One of the 3 following cases
   INTEGER, PARAMETER ::   jpdomcrs_local_full    = 4   !: ( 1  :jpi_crs   , 1  :jpj_crs   )
   INTEGER, PARAMETER ::   jpdomcrs_local_noextra = 5   !: ( 1  :nlci_crs  , 1  :nlcj_crs  )
   INTEGER, PARAMETER ::   jpdomcrs_local_noovlap = 6   !: (nldi_crs:nlei_crs  ,nldj_crs:nlej_crs  )
   INTEGER, PARAMETER ::   jpdomcrs_unknown       = 7   !: No dimension checking
   INTEGER, PARAMETER ::   jpdomcrs_autoglo       = 8   !:
   INTEGER, PARAMETER ::   jpdomcrs_autodta       = 9   !:
   INTEGER            ::   ipdomcrs_local_noovlap_crs, ipdomcrs_local_full_crs, idomcrs_local_noextra_crs

   INTEGER                 ::   idomcrs     ! Type of domain to be written (default = jpdom_local_noovlap)
   INTEGER, DIMENSION(2,5) ::   idompar_crs ! domain parameters:    
   LOGICAL                 ::   llnoov      ! local definition to read overlap


CONTAINS

   SUBROUTINE crs_iom_open( cdname, kiomid, ldwrt, kdom, kiolib )
      !!--------------------------------------------------------------------
      !!                       ***  MODULE crs_iom_open  ***
      !!
      !! ** Purpose : open an input file with NF90 on coarsened grid
      !!---------------------------------------------------------------------
      !! Arguments
      CHARACTER(len=*)       , INTENT(in)           ::   cdname   ! File name
      INTEGER                , INTENT(inout)        ::   kiomid   ! nf90 identifier of the opened file
      LOGICAL                , INTENT(in)           ::   ldwrt    ! read or write the file?
      INTEGER                , INTENT(in), OPTIONAL ::   kdom     ! Type of domain to be written (default = jpdom_local_noovlap)
      INTEGER                , INTENT(in), OPTIONAL ::   kiolib   ! library used to open the file (default = jpnf90)


      !! Save the parent grid information
      jpi_full    = jpi 
      jpj_full    = jpj
      jpim1_full  = jpim1
      jpjm1_full  = jpjm1
      nperio_full = nperio

      npolj_full  = npolj 
      jpnij_full  = jpnij
      narea_full  = narea
      npiglo_full = jpiglo 
      npjglo_full = jpjglo

      nlcj_full   = nlcj 
      nlci_full   = nlci
      nldi_full   = nldi
      nlei_full   = nlei
      nlej_full   = nlej
      nldj_full   = nldj

      !! Switch to coarse grid domain
      jpi    = jpi_crs 
      jpj    = jpj_crs
      jpim1  = jpi_crsm1
      jpjm1  = jpj_crsm1
      nperio = nperio_crs

      npolj  = npolj_crs 
      jpnij  = jpnij_crs
      narea  = narea_crs
      npiglo = jpiglo_crs 
      npjglo = jpjglo_crs 

      nlci   = nlci_crs
      nlcj   = nlcj_crs
      nldi   = nldi_crs
      nlei   = nlei_crs
      nlej   = nlej_crs

      nldj   = nldj_crs


      llnoov = (jpni * jpnj ) == jpnij .AND. .NOT. lk_agrif 
      IF( llnoov ) THEN   ;   idomcrs = jpdomcrs_local_noovlap   ! default definition
      ELSE                ;   idomcrs = jpdomcrs_local_full      ! default definition
      ENDIF
      IF ( PRESENT(kdom) ) idomcrs = kdom

      WRITE(numout,*) 'crs_iom_open. kiomid=', kiomid

      CALL iom_open( cdname, kiomid, ldwrt, idomcrs, kiolib )

      WRITE(numout,*) 'crs_iom_open. after iom_open call kiomid=', kiomid

      !! Return to parent grid domain
      jpi    = jpi_full 
      jpj    = jpj_full
      jpim1  = jpim1_full
      jpjm1  = jpjm1_full
      nperio = nperio_full

      npolj  = npolj_full 
      jpnij  = jpnij_full
      narea  = narea_full
      npiglo = npiglo_full 
      npjglo = npjglo_full 

      nlcj   = nlcj_full 
      nlci   = nlci_full
      nldi   = nldi_full
      nlei   = nlei_full
      nlej   = nlej_full

      nldj   = nldj_full

   END SUBROUTINE crs_iom_open


   SUBROUTINE crs_iom_close( kiomid )
      !!--------------------------------------------------------------------
      !!                       ***  MODULE crs_iom_open  ***
      !!
      !! ** Purpose : open an input file with NF90 on coarsened grid
      !!---------------------------------------------------------------------
      !! Arguments
      INTEGER                , INTENT(inout)           ::   kiomid      ! nf90 identifier of the opened file
      !! Local variable
      CHARACTER(LEN=100)  ::   clinfo   ! info character
      !---------------------------------------------------------------------
      !
      WRITE(numout,*) 'crs_iom_close. kiomid=', kiomid

      CALL iom_close( kiomid )
      WRITE(numout,*) 'crs_iom_close. after iom_open call kiomid=', kiomid

      !    
   END SUBROUTINE crs_iom_close


   SUBROUTINE crs_iom_rstput( kt, kwrite, kiomid, cdvar, pv_r0d, pv_r1d, pv_r2d, pv_r3d, ktype )
      !!--------------------------------------------------------------------
      !!                   ***  SUBROUTINE  crs_iom_rp0d  ***
      !!
      !! ** Purpose : read the time axis cdvar in the file 
      !!--------------------------------------------------------------------
      ! Arguments
      INTEGER          ,                                  INTENT(in) :: kt       ! ocean time-step
      INTEGER          ,                                  INTENT(in) :: kwrite   ! writing time-step
      INTEGER          ,                                  INTENT(in) :: kiomid   ! Identifier of the file 
      CHARACTER(len=*) ,                                  INTENT(in) :: cdvar    ! variable name
      REAL(wp), OPTIONAL,                                 INTENT(in) :: pv_r0d   ! written Od field
      REAL(wp), OPTIONAL, DIMENSION(jpk),                 INTENT(in) :: pv_r1d   ! written 1d field
      REAL(wp), OPTIONAL, DIMENSION(jpi_crs,jpj_crs) ,    INTENT(in) :: pv_r2d   ! written 2d field
      REAL(wp), OPTIONAL, DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(in) :: pv_r3d   ! written 3d field
      INTEGER,  OPTIONAL,                                 INTENT(in) :: ktype    ! variable type (default R8)

      ! Local
      INTEGER               :: ivid     ! variable id
      INTEGER               :: itype    ! variable type

      !! Save the parent grid information
      jpi_full    = jpi 
      jpj_full    = jpj
      jpim1_full  = jpim1
      jpjm1_full  = jpjm1
      nperio_full = nperio

      npolj_full  = npolj 
      jpnij_full  = jpnij
      narea_full  = narea
      npiglo_full = jpiglo 
      npjglo_full = jpjglo

      nlcj_full   = nlcj 
      nlci_full   = nlci
      nldi_full   = nldi
      nlei_full   = nlei
      nlej_full   = nlej
      nldj_full   = nldj

      !! Switch to coarse grid domain
      jpi    = jpi_crs 
      jpj    = jpj_crs
      jpim1  = jpi_crsm1
      jpjm1  = jpj_crsm1
      nperio = nperio_crs

      npolj  = npolj_crs 
      jpnij  = jpnij_crs
      narea  = narea_crs
      npiglo = jpiglo_crs 
      npjglo = jpjglo_crs
 
      nlcj   = nlcj_crs 
      nlci   = nlci_crs
      nldi   = nldi_crs
      nlei   = nlei_crs
      nlej   = nlej_crs
      nldj   = nldj_crs


      IF(     PRESENT(pv_r0d) ) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r0d, ktype )
      ELSEIF( PRESENT(pv_r1d) ) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r1d, ktype )
      ELSEIF( PRESENT(pv_r2d) ) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r2d, ktype )
      ELSEIF( PRESENT(pv_r3d) ) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r3d, ktype )
      ENDIF

      !! Return to parent grid domain
      jpi    = jpi_full 
      jpj    = jpj_full
      jpim1  = jpim1_full
      jpjm1  = jpjm1_full
      nperio = nperio_full

      npolj  = npolj_full 
      jpnij  = jpnij_full
      narea  = narea_full
      npiglo = npiglo_full 
      npjglo = npjglo_full
 
      nlcj   = nlcj_full 
      nlci   = nlci_full
      nldi   = nldi_full
      nlei   = nlei_full
      nlej   = nlej_full
      nldj   = nldj_full

 
   END SUBROUTINE crs_iom_rstput

   SUBROUTINE crs_iom_put( cdvar , pv_r0d, pv_r1d, pv_r2d, pv_r3d )
      !!--------------------------------------------------------------------
      !!                   ***  SUBROUTINE  iom_put  ***
      !!
      !! ** Purpose : read the time axis cdvar in the file 
      !!--------------------------------------------------------------------
      ! Arguments
      CHARACTER(len=*)            , INTENT(in)           ::   cdvar    ! variable name
      REAL(wp)                    , INTENT(in), OPTIONAL ::   pv_r0d   ! written Od field
      REAL(wp), DIMENSION(      :), INTENT(in), OPTIONAL ::   pv_r1d   ! written 1d field
      REAL(wp), DIMENSION(:, :   ), INTENT(in), OPTIONAL ::   pv_r2d   ! written 2d field
      REAL(wp), DIMENSION(:, :, :), INTENT(in), OPTIONAL ::   pv_r3d   ! written 3d field

      ! Local
      INTEGER               :: ivid     ! variable id
      INTEGER               :: itype    ! variable type

      !! Save the parent grid information
      jpi_full    = jpi 
      jpj_full    = jpj
      jpim1_full  = jpim1
      jpjm1_full  = jpjm1
      nperio_full = nperio

      npolj_full  = npolj 
      jpnij_full  = jpnij
      narea_full  = narea
      npiglo_full = jpiglo 
      npjglo_full = jpjglo

      nlcj_full   = nlcj 
      nlci_full   = nlci
      nldi_full   = nldi
      nlei_full   = nlei
      nlej_full   = nlej
      nldj_full   = nldj

      !! Switch to coarse grid domain
      jpi    = jpi_crs 
      jpj    = jpj_crs
      jpim1  = jpi_crsm1
      jpjm1  = jpj_crsm1
      nperio = nperio_crs

      npolj  = npolj_crs 
      jpnij  = jpnij_crs
      narea  = narea_crs
      npiglo = jpiglo_crs 
      npjglo = jpjglo_crs
 
      nlcj   = nlcj_crs 
      nlci   = nlci_crs
      nldi   = nldi_crs
      nlei   = nlei_crs
      nlej   = nlej_crs
      nldj   = nldj_crs

      ! variable definition
      IF(     PRESENT(pv_r0d) ) THEN   ;  CALL iom_put( cdvar, pv_r0d )
      ELSEIF( PRESENT(pv_r1d) ) THEN   ;  CALL iom_put( cdvar, pv_r1d )
      ELSEIF( PRESENT(pv_r2d) ) THEN   ;  CALL iom_put( cdvar, pv_r2d )
      ELSEIF( PRESENT(pv_r3d) ) THEN   ;  CALL iom_put( cdvar, pv_r3d )
      ENDIF

      !! Return to parent grid domain
      jpi    = jpi_full 
      jpj    = jpj_full
      jpim1  = jpim1_full
      jpjm1  = jpjm1_full
      nperio = nperio_full

      npolj  = npolj_full 
      jpnij  = jpnij_full
      narea  = narea_full
      npiglo = npiglo_full 
      npjglo = npjglo_full
 
      nlcj   = nlcj_full 
      nlci   = nlci_full
      nldi   = nldi_full
      nlei   = nlei_full
      nlej   = nlej_full
      nldj   = nldj_full

   END SUBROUTINE crs_iom_put

!#if defined key_iomput

!#endif

END MODULE crs_iom
