MODULE crslbclnk

   !!======================================================================
   !!                       ***  MODULE  crslbclnk  ***
   !!               A temporary solution for lbclnk for coarsened grid.
   !! Ocean        : lateral boundary conditions for grid coarsening
   !!=====================================================================
   !! History :   ! 2012-06  (J. Simeon, G. Madec, C. Ethe)     Original code

   USE dom_oce
   USE crs_dom
   USE lbclnk
   USE par_kind, ONLY: wp
   USE in_out_manager

   PUBLIC crs_lbc_lnk

   CONTAINS

   SUBROUTINE crs_lbc_lnk( cd_type1, psgn, pt2d, pt3d1, pt3d2, cd_type2, cd_mpp )
      !!---------------------------------------------------------------------
      !!                  ***  SUBROUTINE crs_lbc_lnk  ***
      !!
      !! ** Purpose :   set lateral boundary conditions for coarsened grid
      !!
      !! ** Method  :   Swap domain indices from full to coarse domain
      !!                before arguments are passed directly to lbc_lnk.
      !!                Upon exiting, switch back to full domain indices.
      !!----------------------------------------------------------------------
      !! Arguments
      CHARACTER(len=1)                , INTENT(in   )           ::   cd_type1 ! grid type
      REAL(wp)                        , INTENT(in   )           ::   psgn     ! control of the sign

      REAL(wp), DIMENSION(jpi_crs,jpj_crs),     INTENT(inout), OPTIONAL ::   pt2d  ! 2D array on which the lbc is applied
      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(inout), OPTIONAL ::   pt3d1 ! 3D array on which the lbc is applied
      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(inout), OPTIONAL ::   pt3d2      

      CHARACTER(len=1)                , INTENT(in   ), OPTIONAL ::   cd_type2 ! grid type
      CHARACTER(len=3)                , INTENT(in   ), OPTIONAL ::   cd_mpp    ! MPP only (here do nothing)

      !!----------------------------------------------------------------------
      !! Save the parent grid information
      jpi_full    = jpi 
      jpj_full    = jpj
      jpim1_full  = jpim1
      jpjm1_full  = jpjm1
      nperio_full = nperio

      npolj_full  = npolj 
      jpnij_full  = jpnij
      narea_full  = narea
      npiglo_full = jpiglo 
      npjglo_full = jpjglo

      nlcj_full   = nlcj 
      nlci_full   = nlci
      nldi_full   = nldi
      nlei_full   = nlei
      nlej_full   = nlej
      nldj_full   = nldj

!     jpni_full = jpni

      !! Switch to coarse grid domain
      jpi    = jpi_crs 
      jpj    = jpj_crs
      jpim1  = jpi_crsm1
      jpjm1  = jpj_crsm1
      nperio = nperio_crs

      npolj  = npolj_crs 
      jpnij  = jpnij_crs
      narea  = narea_crs
      npiglo = jpiglo_crs 
      npjglo = jpjglo_crs 

      nlcj   = nlcj_crs 
      nlci   = nlci_crs
      nldi   = nldi_crs
      nlei   = nlei_crs
      nlej   = nlej_crs
      nldj   = nldj_crs
!     jpni = jpni_crs

      IF ( PRESENT( pt2d ) )  THEN

         IF ( PRESENT( cd_mpp ) ) THEN
            CALL lbc_lnk( pt2d, cd_type1, psgn, cd_mpp )
         ELSE
            CALL lbc_lnk( pt2d, cd_type1, psgn )
         ENDIF
        
      ELSEIF ( PRESENT(pt3d1) .AND. PRESENT(pt3d2) )  THEN

         CALL lbc_lnk( pt3d1, cd_type1, pt3d2, cd_type2, psgn )

      ELSEIF ( PRESENT(pt3d1) ) THEN

         IF ( PRESENT( cd_mpp ) ) THEN
            CALL lbc_lnk( pt3d1, cd_type1, psgn, cd_mpp )
         ELSE
            CALL lbc_lnk( pt3d1, cd_type1, psgn )
         ENDIF

      ELSE

         WRITE(numout, *) 'crslbclnk. Must supply a 2d or 3d field'
         STOP

      ENDIF

      !! Return to parent grid domain
      jpi    = jpi_full 
      jpj    = jpj_full
      jpim1  = jpim1_full
      jpjm1  = jpjm1_full
      nperio = nperio_full

      npolj  = npolj_full 
      jpnij  = jpnij_full
      narea  = narea_full
      npiglo = npiglo_full 
      npjglo = npjglo_full 

      nlcj   = nlcj_full 
      nlci   = nlci_full
      nldi   = nldi_full
      nlei   = nlei_full
      nlej   = nlej_full
      nldj   = nldj_full

!     jpni = jpni_full
   END SUBROUTINE crs_lbc_lnk


END MODULE crslbclnk
